module mtests
   use constants
   use options
   use mfunctions
   use mrestore
   use save
   use ltest
   implicit none

contains

   subroutine pwtest(nleg,rank,ok)
      implicit none
      integer :: i,j
      integer :: nleg,rank,diff
      real(ki), dimension(4) :: e1, e2
      real(ki) :: pwval
      complex(ki), dimension(4) :: e3, e4
      complex(ki) :: pwt
      complex(ki), dimension(4) :: pwtv
      logical :: ok

      ok=.true.

      diff = nleg-rank
      pwt=czip
 
      if(verbosity.ge.3)then
          write(20,*) '------------------------------------'
          write(20,*) 'PoWer test'
      endif

      if (diff.eq.0) then
         pwtv=(/ czip, czip, czip, czip /)
         
         do j =1,max1
            do i=1,4
               e1(i)=sav1e1(j,i)
               e2(i)=sav1e2(j,i)
               e3(i)=sav1e3(j,i)
               e4(i)=sav1e4(j,i)
            enddo
            do i=1,4
               pwtv(i)=pwtv(i)+savc1(j,1)*e1(i)+savc1(j,2)*e2(i) &
              &               +savc1(j,3)*e3(i)+savc1(j,4)*e4(i)
            enddo
         enddo
 
         pwval =  maxval(abs(pwtv))
         if(verbosity.ge.3) write(20,*) 'pwtest = ', pwval

         if (pwval.ge.pwlimit) then
            ok=.false.
            if(verbosity.gt.0) then
               write(20,*) ' POWERTEST FAILED! ' 
               write(20,*) ' point above discarded'
            endif
         else
           if(verbosity.ge.3) write(20,*) ' PowerTest passed! ' 
         endif

         if(verbosity.ge.3) write(20,*) '-----------------------------------'
    
         return

      elseif (diff.eq.1) then
         pwt=czip
         do i =1,6
            pwt=pwt+savc1(i,0)
         enddo
          
         pwval =  abs(pwt)
         if(verbosity.ge.3) write(20,*) 'pwtest = ', pwval
         if (pwval.ge.pwlimit) then
            ok=.false.
            if(verbosity.gt.0) then
               write(20,*) ' POWERTEST FAILED! ' 
               write(20,*) 'point above discarded'
            endif
         else
            if(verbosity.ge.3) write(20,*) ' PowerTest passed! ' 
         endif
         if(verbosity.ge.3) write(20,*) '------------------------------------'
         
         return

      elseif (diff.eq.2) then
         pwt=czip
         do i =1,max2
            pwt=pwt+savc2(i,0)
         enddo


         pwval =  abs(pwt)
         if(verbosity.ge.3) write(20,*) 'pwtest = ', pwval
         if (pwval.ge.pwlimit) then
            ok=.false.
            if(verbosity.gt.0) then
               write(20,*) ' POWERTEST FAILED! ' 
               write(20,*) 'point above discarded'
            endif
         else
            if(verbosity.ge.3) write(20,*) ' PowerTest passed! ' 
         endif
         if(verbosity.ge.3) write(20,*) '------------------------------------'
             
         return
          
      else          
         pwt=czip
         do i =1,max3
            pwt=pwt+savc3(i,0)
         enddo
    
         pwval =  abs(pwt)
         if(verbosity.ge.3) write(20,*) 'pwtest = ', pwval
         if (pwval.ge.pwlimit) then 
            ok=.false.
            if(verbosity.gt.0) then
               write(20,*) ' POWERTEST FAILED! ' 
               write(20,*) 'point above discarded'
            endif
         else
            if(verbosity.ge.3) write(20,*) ' PowerTest passed! ' 
         endif
         if(verbosity.ge.3) write(20,*) '------------------------------------'
         
         return

       endif

   end subroutine

   subroutine nntest(numeval,q1,mu2,nleg,Vi,msq,ok)
      implicit none
      integer :: i, i1, i2, i3, i4, i5, ncut
      integer :: dicut5, dicut4, dicut3, dicut2, dicut1,nleg
      complex(ki) :: dens1,dens2,dens3,dens4,dens5,xneval
      complex(ki), dimension(4) :: q1
      complex(ki) :: resi5, resi4, resi3, resi2 ,resi1, resitot, ztest
      logical :: ok

      interface
         function     numeval(ncut, Q, mu2)
            use precision
            implicit none
            integer, intent(in) :: ncut
            complex(ki), dimension(4), intent(in) :: Q
            real(ki), intent(in) :: mu2
            complex(ki) :: numeval
          end function numeval
      end interface
     
      real(ki), dimension(0:nleg-1) :: msq
      real(ki), dimension(0:nleg-1,4) :: Vi

      real(ki) :: mu2

      ok = .true.

      resi1=czip
      resi2=czip
      resi3=czip
      resi4=czip
      resi5=czip

      ztest=czip
      xneval=czip

      if (nleg.eq.5) then
         
         resi5=res5(1,mu2)
  
         goto 11

      elseif (nleg.eq.4) then
        
         resi4=Res4(1,q1,mu2)
       
         goto 21

      elseif (nleg.eq.3) then
    
         resi3=Res3(1,q1,mu2)
        
         goto 31

      elseif (nleg.eq.2) then
       
         resi2=Res2(1,q1,mu2)
       
         goto 41

      else


!--- Contributo dei pentuple cuts
      dicut5=1

      do i5=4,nleg-1
       do i4=3,i5-1
        do i3=2,i4-1
         do i2=1,i3-1
          do i1=0,i2-1

      dens5=cone

      do i=0,nleg-1 
      if (((i).eq.(i1)).or.((i).eq.(i2)) &
     & .or.((i).eq.(i3)).or.((i).eq.(i4)).or.((i).eq.(i5))) then 
      cycle
      else
      dens5=dens5*denevalmu2(nleg,i,q1,Vi,msq,mu2)
      endif
      enddo
      resi5=resi5+dens5*res5(dicut5,mu2)
      dicut5=dicut5+1
          enddo
         enddo
        enddo
       enddo
      enddo

      endif

 11   continue


!--- Contributo dei quadruple cuts
      dicut4=1
      do i4=3,nleg-1
       do i3=2,i4-1
        do i2=1,i3-1
         do i1=0,i2-1

      dens4=cone

      do i=0,nleg-1
      if (((i).eq.(i1)).or.((i).eq.(i2)) &
     &.or.((i).eq.(i3)).or.((i).eq.(i4))) then
      cycle
      else
      dens4=dens4*denevalmu2(nleg,i,q1,Vi,msq,mu2)
      endif
      enddo
                     
      resi4=resi4+dens4*Res4(dicut4,q1,mu2)
      dicut4=dicut4+1
         enddo
        enddo
       enddo
      enddo

 21    continue

!--- Contributo dei Triple cuts
       dicut3=1
       do i3=2,nleg-1
        do i2=1,i3-1
         do i1=0,i2-1

       dens3=cone
                
       do i=0,nleg-1
       if (((i).eq.(i1)).or.((i).eq.(i2)).or.((i).eq.(i3))) then
       cycle
       else
       dens3=dens3*denevalmu2(nleg,i,q1,Vi,msq,mu2)
       endif
       enddo

       resi3=resi3+dens3*Res3(dicut3,q1,mu2)
       dicut3=dicut3+1
         enddo
        enddo
       enddo

 31   continue

!--- Contributo dei Double cuts
      dicut2=1
      do i2=1,nleg-1
       do i1=0,i2-1

      dens2=cone
   
      do i=0,nleg-1
      if (((i).eq.(i1)).or.((i).eq.(i2))) then
      cycle
      else
      dens2=dens2*denevalmu2(nleg,i,q1,Vi,msq,mu2)
      endif
      enddo

      resi2=resi2+dens2*Res2(dicut2,q1,mu2)
      dicut2=dicut2+1

       enddo
      enddo

 41   continue

!- Contribution of the single cut

      dicut1=1
      do i1=0,nleg-1
      dens1=cone
         
      do i=0,nleg-1
      if (i.eq.i1) then
      cycle
      else
      dens1=dens1*denevalmu2(nleg,i,q1,Vi,msq,mu2)
      endif
      enddo
      resi1=resi1+dens1*Res1(dicut1,q1)
      dicut1=dicut1+1
      enddo

      xneval=numeval(ncut,q1,mu2)
      resitot=resi5+resi4+resi3+resi2+resi1
      ztest=xneval-resitot

      if(verbosity.ge.3)then
         write(20,*) '--------------------------------------------'
         write(20,*) 'N=N test for'
         write(20,*) 'q(0) = ', q1(4)
         write(20,*) 'q(1) = ', q1(1)
         write(20,*) 'q(2) = ', q1(2)
         write(20,*) 'q(3) = ', q1(3)
         write(20,*) 'and mu2 = ', mu2
         write(20,*) 'N    calculated', xneval
         write(20,*) 'N reconstructed', resitot
         write(20,*) ' '
         write(20,*) 'difference = ', ztest
         write(20,*) 'rel.diff.  = ', ztest/xneval
         write(20,*) '--------------------------------------------'
      endif
      
      if (abs(ztest/xneval).gt.nnlimit) then
         ok=.false.
         if (verbosity.gt.0) write(20,*) 'N=N test FAILED'
      endif
      return

    end subroutine nntest

   subroutine lnntest4(numeval,cut4,c4,qt,p0,k3,e3,e4,ok)
      implicit none
      integer cut4,i
      complex(ki), dimension(4) :: pm, qt, e3, e4
      complex(ki) :: c4(0:4), resi5t, dens4t, test4, poli4, reldif
      real(ki), dimension(4) :: p0, k3
      logical :: ok
      common/t4/resi5t,dens4t
      interface
         function     numeval(ncut, Q, mu2)
            use precision
            implicit none
            integer, intent(in) :: ncut
            complex(ki), dimension(4), intent(in) :: Q
            real(ki), intent(in) :: mu2
            complex(ki) :: numeval
         end function numeval
      end interface

      ok = .true.

      do i=1,4
         pm(i)=qt(i)+p0(i)
      enddo
      if     (imeth.eq.'diag') then
      test4=(numeval(cut4,qt,half)-resi5t)/dens4t
      elseif (imeth.eq.'tree') then
      test4= numeval(cut4,qt,half)-resi5t /dens4t
      endif
      poli4=poly4(c4,pm,half,k3,e3,e4)
      reldif=(test4-poli4)/poli4
      if(verbosity.ge.3)then
         write(20,*) ' LOCAL N=N TEST - BOX'
         write(20,*) 'cut4    =',cut4
         write(20,*) 'test4   =',test4
         write(20,*) 'poli4   =',poli4
         write(20,*) 'diff4   =',test4-poli4
         write(20,*) 'rel.dif4=',abs(reldif)
      endif

      if (abs(reldif).gt.lnnlimit4) then
         ok=.false.
         if (verbosity.gt.0) write(20,*) 'LOCAL N=N test FAILED for the BOX',cut4
      endif
      return

    end subroutine lnntest4


   subroutine lnntest3(numeval,cut3,c3,qt,p0,e3,e4,ok)
     implicit none
     integer cut3,i
      complex(ki), dimension(4) :: pm, qt, e3, e4
      complex(ki) :: c3(0:9), resit, dens3t, test3, poli3, reldif
      real(ki), dimension(4) :: p0 
      logical :: ok  
      interface
         function     numeval(ncut, Q, mu2)
           use precision
           implicit none
           integer, intent(in) :: ncut
           complex(ki), dimension(4), intent(in) :: Q
           real(ki), intent(in) :: mu2
           complex(ki) :: numeval
         end function numeval
      end interface
      common/t3/resit,dens3t

      ok = .true.

      do i=1,4
         pm(i)=qt(i)+p0(i)
      enddo
      if     (imeth.eq.'diag') then
      test3=(numeval(cut3,qt,half)-resit)/dens3t
      elseif (imeth.eq.'tree') then
      test3= numeval(cut3,qt,half)-resit /dens3t
      endif
      poli3=poly3(c3,pm,half,e3,e4)
      reldif=(test3-poli3)/poli3
      if(verbosity.ge.3)then
         write(20,*) ' LOCAL N=N TEST - triangle'
         write(20,*) " cut3   =",cut3
         write(20,*) "test3   =",test3
         write(20,*) "poli3   =",poli3
         write(20,*) "diff3   =",test3-poli3
         write(20,*) "rel.dif3=",abs(reldif)
      endif
         
      if (abs(reldif).gt.lnnlimit3) then
         ok=.false.
         if (verbosity.gt.0) write(20,*) 'LOCAL N=N test FAILED for the 3-cut',cut3
      endif
      return
    end subroutine lnntest3

    subroutine lnntest2(numeval,cut2,c2,qt,p0,e2,e3,e4,ok)
      implicit none
      integer cut2,i
      complex(ki), dimension(4) :: pm, qt, e3, e4
      complex(ki) :: c2(0:9), resit, dens2t, test2, poli2, reldif
      real(ki), dimension(4) :: p0, e2
      logical :: ok
      interface
         function     numeval(ncut, Q, mu2)
            use precision
            implicit none
            integer, intent(in) :: ncut
            complex(ki), dimension(4), intent(in) :: Q
            real(ki), intent(in) :: mu2
            complex(ki) :: numeval
         end function numeval
      end interface
      common/t2/resit,dens2t

      ok = .true.

      do i=1,4
         pm(i)=qt(i)+p0(i)
      enddo
      if     (imeth.eq.'diag') then
         test2=(numeval(cut2,qt,half)-resit)/dens2t
      elseif (imeth.eq.'tree') then
         test2= numeval(cut2,qt,half)-resit /dens2t
      endif
      poli2=poly2(c2,pm,half,e2,e3,e4)
      reldif=(test2-poli2)/poli2
      
      if(verbosity.ge.3)then
         write(20,*) ' LOCAL N=N TEST - Bubble'
         write(20,*) " cut2   =",cut2
         write(20,*) "test2   =",test2
         write(20,*) "poli2   =",poli2
         write(20,*) "diff2   =",test2-poli2
         write(20,*) "rel.dif2=",abs(reldif)
      endif

      if (abs(reldif).gt.lnnlimit2) then
         ok=.false.
         if (verbosity.gt.0) write(20,*) 'LOCAL N=N test FAILED for the 2-cut',cut2
      endif
      return
    end subroutine lnntest2


      subroutine lnntest1(numeval,cut1,c1,qt,p0,e1,e2,e3,e4,ok)
      implicit none
      integer cut1,i
      complex(ki), dimension(4) :: pm, qt, e3, e4
      complex(ki) :: c1(0:4), resit, dens1t, test1, poli1, reldif
      real(ki), dimension(4) :: p0, e1, e2
      logical :: ok
      interface
         function     numeval(ncut, Q, mu2)
            use precision
            implicit none
            integer, intent(in) :: ncut
            complex(ki), dimension(4), intent(in) :: Q
            real(ki), intent(in) :: mu2
            complex(ki) :: numeval
         end function numeval
      end interface
      common/t1/resit,dens1t
     
      ok = .true.

      do i=1,4
         pm(i)=qt(i)+p0(i)
      enddo
      if     (imeth.eq.'diag') then
      test1=(numeval(cut1,qt,half)-resit)/dens1t
      elseif (imeth.eq.'tree') then
      test1= numeval(cut1,qt,half)-resit /dens1t
      endif
      poli1=poly1(c1,pm,e1,e2,e3,e4)
      reldif=(test1-poli1)/poli1

      if (verbosity.ge.3) then
         write(20,*) ' LOCAL N=N TEST - Tadpole'         
         write(20,*) " cut1   =",cut1
         write(20,*) "test1   =",test1
         write(20,*) "poli1   =",poli1
         write(20,*) "diff1   =",test1-poli1
         write(20,*) "rel.dif1=",abs(reldif)
      endif

      if (abs(reldif).gt.lnnlimit1) then
         ok=.false.
         if (verbosity.gt.0) write(20,*) 'LOCAL N=N test FAILED for the 1-cut',cut1
      endif
      return
    end subroutine lnntest1


end module mtests
